@@ -1,13 +1,13 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Callable, cast
+from typing import TYPE_CHECKING, Any, Callable, cast
 
 from ._base import BaseEvent, BaseType, base_types, event_types
 from ._pycrdt import Subscription
 from ._pycrdt import Text as _Text
 from ._pycrdt import TextEvent as _TextEvent
 
-if TYPE_CHECKING:  # pragma: no cover
+if TYPE_CHECKING:
     from ._doc import Doc
 
 
@@ -233,18 +233,67 @@ class Text(BaseType):
         """Remove the entire range of characters."""
         del self[:]
 
-    def insert(self, index: int, value: str) -> None:
+    def insert(self, index: int, value: str, attrs: dict[str, Any] | None = None) -> None:
         """
-        Inserts a string at a given index in the text.
+        Inserts a string at a given index in the text, with optional attributes.
+        ```py
         Doc()["text"] = text = Text("Hello World!")
         text.insert(5, ",")
         assert text == "Hello, World!"
+        ```
 
         Args:
             index: The index where to insert the string.
             value: The string to insert in the text.
+            attrs: The formatting of attributes to apply.
         """
-        self[index:index] = value
+        with self.doc.transaction() as txn:
+            self._forbid_read_transaction(txn)
+            self.integrated.insert(
+                txn._txn, index, value, iter(attrs.items()) if attrs is not None else None
+            )
+
+    def insert_embed(self, index: int, value: Any, attrs: dict[str, Any] | None = None) -> None:
+        """
+        Insert a value as an embed at a given index in the text, with optional attributes.
+
+        Args:
+            index: The index where to insert the embedded content.
+            value: The value to embed.
+            attrs: The formatting attributes to apply.
+        """
+        with self.doc.transaction() as txn:
+            self._forbid_read_transaction(txn)
+            self.integrated.insert_embed(
+                txn._txn, index, value, iter(attrs.items()) if attrs is not None else None
+            )
+
+    def format(self, start: int, stop: int, attrs: dict[str, Any]) -> None:
+        """
+        Applies formatting attributes to a section of text between given start and stop indices.
+
+        Args:
+            start: The index at which to start applying attributes (included).
+            stop: The index at which to stop applying attributes (excluded).
+            attrs: The formatting attributes to apply.
+        """
+        with self.doc.transaction() as txn:
+            self._forbid_read_transaction(txn)
+            start, stop = self._check_slice(slice(start, stop))
+            length = stop - start
+            if length > 0:
+                self.integrated.format(txn._txn, start, length, iter(attrs.items()))
+
+    def diff(self) -> list[tuple[Any, dict[str, Any] | None]]:
+        """
+        Returns:
+            A list of formatted chunks that the current text corresponds to.
+                Each list item is a tuple containing the chunk's content and formatting attributes.
+                The content is usually the text as a string, but may be other data for embedded
+                objects.
+        """
+        with self.doc.transaction() as txn:
+            return self.integrated.diff(txn._txn)
 
     def observe(self, callback: Callable[[TextEvent], None]) -> Subscription:
         """
